package com.ruoyi.tast.controller;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.tast.domain.*;
import com.ruoyi.tast.mysingleton.MySingletonHashMap;
import com.ruoyi.tast.result.Result;
import com.ruoyi.tast.service.CarService;
import com.ruoyi.tast.socket.Websocket;
import lombok.extern.log4j.Log4j2;
import org.checkerframework.checker.units.qual.C;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;



@RequestMapping("/car")
@RestController
@Log4j2
public class CarController {

    @Autowired
    private CarService carService;
    @Autowired
    private HttpServletRequest request;

    /**
     * 汽车列表展示
     * @return
     */
    @PostMapping("/carList")
    public Result<List<Car>> carList(@RequestBody Car car){
        log.info("功能名称，汽车列表展示，请求URI：{}，请求方式：{}",request.getRequestURI(),request.getMethod());
        List<Car> list=carService.carList(car);
        Result<List<Car>> result= Result.success(list);
        log.info("功能名称，汽车列表展示，请求URI：{}，请求方式：{}，响应结果：{}",request.getRequestURI(),request.getMethod(),JSONObject.toJSONString(result));
        return result;
    }

    /**
     * 查询汽车类型下拉框
     * @return
     */
    @PostMapping("/findByType")
    public Result<List<Type>> findByType(){
        log.info("功能名称，查询汽车类型下拉框，请求URI：{}，请求方式：{}",request.getRequestURI(),request.getMethod());
        List<Type> list=carService.findByType();
        Result<List<Type>> result = Result.success(list);
        log.info("功能名称，查询汽车类型下拉框，请求URI：{}，请求方式：{}，响应结果：{}",request.getRequestURI(),request.getMethod(),JSONObject.toJSONString(result));
        return result;
    }

    /**
     * 选择电子围栏
     * @return
     */
    @PostMapping("/findByFence")
    public Result<List<ElectronicFenceSetting>> findByFence(){
        log.info("功能名称，选择电子围栏，请求URI：{}，请求方式：{}",request.getRequestURI(),request.getMethod());
        List<ElectronicFenceSetting> list=carService.findByFence();
        Result<List<ElectronicFenceSetting>> result = Result.success(list);
        log.info("功能名称，选择电子围栏，请求URI：{}，请求方式：{}，响应结果：{}",request.getRequestURI(),request.getMethod(),JSONObject.toJSONString(result));
        return result;
    }

    /**
     * 选择企业
     * @return
     */
    @PostMapping("/findByFirm")
    public Result<List<Firm>> findByFirm(){
        log.info("功能名称，选择企业，请求URI：{}，请求方式：{}",request.getRequestURI(),request.getMethod());
        List<Firm> list=carService.findByFirm();
        Result<List<Firm>> result = Result.success(list);
        log.info("功能名称，选择企业，请求URI：{}，请求方式：{}，响应结果：{}",request.getRequestURI(),request.getMethod(),JSONObject.toJSONString(result));
        return result;
    }

    /**
     * 添加车辆
     * @param car
     * @return
     */
    @PostMapping("/addCar")
    public Result addCar(@RequestBody Car car){
        log.info("功能名称，添加车辆，请求URI：{}，请求方式：{}，请求参数：{}",request.getRequestURI(),request.getMethod(),JSONObject.toJSONString(car));
        carService.addCar(car);
        Result result=Result.success();
        log.info("功能名称，添加车辆，请求URI：{}，请求方式：{}，响应结果：{}",request.getRequestURI(),request.getMethod(),JSONObject.toJSONString(result));
        return result;
    }

    /**
     * 删除汽车
     * @param carId
     * @return
     */
    @PostMapping("/deleteCar/{carId}")
    public Result deleteCar(@PathVariable Integer carId){
        log.info("功能名称，删除汽车，请求URI：{}，请求方式：{}，请求参数：{}",request.getRequestURI(),request.getMethod(),carId);
        carService.deleteCar(carId);
        Result result=Result.success();
        log.info("功能名称，删除汽车，请求URI：{}，请求方式：{}，响应结果：{}",request.getRequestURI(),request.getMethod(),JSONObject.toJSONString(result));
        return result;
    }

    /**
     * 编辑车辆信息回显
     * @param carId
     * @return
     */
    @PostMapping("/findByCar/{carId}")
    public Result<Car> findByCar(@PathVariable Integer carId){
        log.info("功能名称，编辑车辆信息回显，请求URI：{}，请求方式：{}，请求参数：{}",request.getRequestURI(),request.getMethod(),carId);
        Car car=carService.findByCar(carId);
        Result<Car> result=Result.success(car);
        log.info("功能名称，编辑车辆信息回显，请求URI：{}，请求方式：{}，响应结果：{}",request.getRequestURI(),request.getMethod(),JSONObject.toJSONString(result));
        return result;
    }

    /**
     * 修改车辆
     * @param car
     * @return
     */
    @PostMapping("/updateCar")
    public Result updateCar(@RequestBody Car car){
        log.info("功能名称，修改车辆，请求URI：{}，请求方式：{}，请求参数：{}",request.getRequestURI(),request.getMethod(),JSONObject.toJSONString(car));
        carService.updateCar(car);
        Result result=Result.success();
        log.info("功能名称，修改车辆，请求URI：{}，请求方式：{}，响应结果：{}",request.getRequestURI(),request.getMethod(),JSONObject.toJSONString(result));
        return result;
    }


    @Autowired
    private RabbitTemplate rabbitTemplate;

    @PostMapping("/send")
    public String sendMail(){
        rabbitTemplate.convertAndSend("emailQueue","hello");
        System.out.println("发送成功");
        return "发送成功";
    }


    /**
     * 获取实时数据
     * @return
     */
    @PostMapping("/report/{vinCode}")
    public Result<VehicleMessage> report(@PathVariable String vinCode){
        log.info("功能名称，获取实时数据，请求URI：{}，请求方式：{}",request.getRequestURI(),request.getMethod());

        VehicleMessage vehicleMessage=carService.getcar(vinCode);

        Result<VehicleMessage> result=Result.success(vehicleMessage);

//        Websocket.sendMessage(String.valueOf(vehicleMessage));
        System.out.println("+++++++++++++++++++++++++"+vehicleMessage);
        log.info("功能名称，获取实时数据，请求URI：{}，请求方式：{}，响应结果：{}",request.getRequestURI(),request.getMethod(),JSONObject.toJSONString(result));
        return result;
    }

    /**
     * websocket给前台推送消息
     * @return
     */
    @GetMapping("/send")
    public Result<List<Message>> send() {
        log.info("功能名称，websocket给前台推送消息，请求URI：{}，请求方式：{}", request.getRequestURI(), request.getMethod());
        List<Message> messageList = carService.sendCar();

        Result<List<Message>> result = Result.success(messageList);

        System.out.println("++++++"+messageList.size());

        Websocket.sendMessage(String.valueOf(messageList),messageList.size());

        log.info("功能名称，websocket给前台推送消息，请求URI：{}，请求方式：{},响应结果：{}", request.getRequestURI(), request.getMethod(), JSONObject.toJSONString(result));

        return result;
    }

}
